@@ -1,6 +1,7 @@
 from .base import HWSystem
 from .base import Wire
 from .base import Logic
+from .logic import Waveform
 
 class Simulator:
 
@@ -46,6 +47,7 @@ class Simulator:
         
         self.clockables = []
         self.propagatables = []
+        self.wave_scopes = []
        
         leaves = self.sys.allLeaves()
         
@@ -54,6 +56,8 @@ class Simulator:
                 self.clockables.append(leaf)
             if (leaf.isPropagatable()):
                 self.propagatables.append(leaf)
+            if (isinstance(leaf, Waveform)):
+                self.wave_scopes.append(leaf)
                 
         # Now sort the propagatables list
         anyChange = True 
@@ -152,4 +156,20 @@ class Simulator:
         
     def _notifyListeners(self):
         for listener in self.listeners:
-            listener.simulatorUpdated()
\ No newline at end of file
+            listener.simulatorUpdated()
+    
+    def get_waveform(self, name:str = "", with_ck = True):
+      signals = [wf.get_wave_raw() for wf in self.wave_scopes]
+      if (with_ck):
+          cklen = len(signals[0]["wave"])-1
+          signals.insert(0, {"name": "CK", "wave": "P" + "."*cklen})
+
+      waveform = {
+        "signal": signals,
+        "head":{
+          "text": name,
+          "tock": 0,
+        }
+      }
+      
+      return waveform
\ No newline at end of file
